/* ESP32_OpenWeatherMap2

   Időjárási adatok lekérése az OpenWeatherMap szerverről
   (ingynes regisztrációval beszerezheztó API kulcs kell hozzá.
   A kapott adatokat megszűrjük, csak a Weather[0] szekció id
   és a main szekció temp, pressure és humidity adatait tartjuk
   meg. Az 'id' értékétől függően a megfelelő ikont kirajzoljuk
   és az időjárási adatokat kiíratjuk az OLED kijelzőn.

   Fentiek mellett egy SNTP szerverről lekért pontos időt és dátumot is
   kiíratjuk az OLED kijelző alső felén.

   Az I2C OLED kijelző SSD1306 vagy SH1106 lehet, 128x64 felbontással.
   A felhasznált OLED_SSD1306_SH1106 könyvtár szoftveresen kezeli az
   I2C kommunikációt!

   Tested with Arduino IDE 1.8.19 and ESP32 Arduino core 2.0.2
*/
#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <time.h>
#include "oled.h"
#include "icons.h"
#include "secrets.h"

//OLED display = OLED(21,22,NO_RESET_PIN,0x3C,128,64, false); // SSD1306
OLED display = OLED(21, 22, NO_RESET_PIN, 0x3C, 128, 64, true); // SH1106

StaticJsonDocument<500> doc;
StaticJsonDocument<200> filter;
String server_url = "http://api.openweathermap.org/data/2.5/weather?q=Debrecen&mode=json&APPID=" + OPENWEATHERMAP_APPID;
static char msg[30];      // character buffer

// TimeZone rule for Europe/Budapest including daylight adjustment rules (optional)
// See at: https://leo.leung.xyz/wiki/Timezone
const char* time_zone = "CET-1CEST,M3.5.0,M10.5.0/3";
const char* ntpServer = "hu.pool.ntp.org";
struct tm timeinfo;
unsigned long then1=0, then2=0;
#define INTERVAL1    30000UL       // Half minute intervals for OpenWeatherMap API calls
#define INTERVAL2     1000UL       // One second intervals for the clock update

void setup () {
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID,WIFI_PASS);
  Serial.print("Connecting to ");
  Serial.print(WIFI_SSID);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  configTzTime(time_zone, ntpServer);
  display.begin();
  display.clear();
  display.display();
  // Prepare filter
  filter["weather"][0]["id"] = true;
  filter["main"]["temp"] = true;
  filter["main"]["pressure"] = true;
  filter["main"]["humidity"] = true;
}

void loop() {
  unsigned long now = millis();
  if (WiFi.status() == WL_CONNECTED) {       // Check WiFi connection status
    if ((now - then1) > INTERVAL1) {         // Every 30 sec...
      HTTPClient http;                       // Declare an object of class HTTPClient
      http.begin(server_url);                // Specify request destination
      int httpCode = http.GET();             // Send the request
      if (httpCode > 0) {                    // Check the returning code
        String payload = http.getString();   // Get the request response payload
        //       Serial.println(payload);           // Print the response payload
        DeserializationError error = deserializeJson(doc, payload, DeserializationOption::Filter(filter));
        // Test if parsing succeeds.
        if (error) {
          Serial.print(F("deserializeJson() failed: "));
          Serial.println(error.f_str());
        } else {
          int id = doc["weather"][0]["id"];
          float tempK = (doc["main"]["temp"]);
          int hPa = doc["main"]["pressure"];
          int relHum = doc["main"]["humidity"];
          display.draw_rectangle(0, 0, 127, 31, OLED::SOLID, OLED::BLACK); // Clear upper half of display
          display.draw_string(2, 0, "DEBRECEN");
          display.printf(2, 8, "T: %.1f°C", tempK-273.16);
          display.printf(2, 16, "Hum: %d %%", relHum);
          display.printf(2, 24, "P: %d hPa", hPa);
          drawIcon(96, 0, 32, 32, id, OLED::WHITE);
          display.display();
          serializeJsonPretty(doc, Serial);
        }
      }
      http.end();                                                         //Close connection
      then1 += INTERVAL1;
    }
    if ((now - then2) > INTERVAL2) {                                      // In every second
      if (getLocalTime(&timeinfo)) {
        memset(msg, 0, sizeof(msg));
        display.draw_rectangle(0, 32, 127, 63, OLED::SOLID, OLED::BLACK); // Clear lower half of display
        strftime (msg, 16, "%Y-%b-%d %a ", &timeinfo);
        display.draw_string(15, 35, msg);                                 // Display date
        strftime (msg, 10, "%T ", &timeinfo);                             // Display time
        display.draw_string(8, 45, msg, OLED::DOUBLE_SIZE);
        display.display();                                                // Refresh screen
      }
      then2 += INTERVAL2;
    }
  }
}

void drawIcon(int16_t x, int16_t y, int16_t w, int16_t h, int id, OLED::tColor color) {
  const unsigned char* bitmap;
  int16_t byteWidth = (w + 7) / 8; // Bitmap scanline pad = whole byte
  uint8_t byte = 0;
  if (id == 800)
    bitmap = bitmap_sun;
  else if ((id >= 200) && (id < 300))
    bitmap = bitmap_lightning;
  else if ((id > 800) && (id < 900))
    bitmap = bitmap_cloud;
  else bitmap = bitmap_umbrella;
  for (int16_t j = 0; j < h; j++, y++) {
    for (int16_t i = 0; i < w; i++) {
      if (i & 7) {
        byte <<= 1;
      } else {
        byte = (*(const unsigned char *)(&bitmap[j * byteWidth + i / 8]));
      }
      if (byte & 0x80) display.draw_pixel(x + i, y, color);
    }
  }
}
